package com.cesgroup.kingkong.service.impl;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Pattern;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.cesgroup.common.utils.PatentTypeEnum;
import com.cesgroup.common.utils.TypeEnum;
import com.cesgroup.kingkong.mapper.EsMapper;


@Service
public class QueryService {

	@Autowired
	private EsMapper esMapper;

	/**
	 * 通过ID获取数据
	 * 
	 * @param index  索引，类似数据库
	 * @param type   类型，类似表
	 * @param id     数据ID
	 * @param fields 需要显示的字段，逗号分隔（缺省为全部字段）
	 * @return Map<String,Object>
	 */
	public Map<String, Object> getById(String id) {
		Map<String, Object> result = new HashMap<String, Object>();
		// 首先id满足条件，通过Id去查
		String VALID_NAME = "^[A-Za-z][A-Za-z].*[.][0-9]$";
		String PRE_VALID_NAME = "^[A-Za-z][A-Za-z].*$";
		long ti = System.currentTimeMillis();
		System.out.println("开始查询："+ti);
		if (isValidId(id, VALID_NAME)) {
			if ("zl".equalsIgnoreCase(id.substring(0, 2))) {
				id = "CN" + id.substring(2, id.length());
			}
		} else if (isValidId(id, PRE_VALID_NAME)) {
			if ("zl".equalsIgnoreCase(id.substring(0, 2))) {
				id = "CN" + id.substring(2, id.length());
			}
			if (id.lastIndexOf(".") != -1) {
				id = id.substring(0, id.lastIndexOf("."));
			}

		} else {
			// 如果都不符合，先切割点之前的数字，然后拼接成CN+数字方式
			if (id.lastIndexOf(".") != -1) {
				id = id.substring(0, id.lastIndexOf("."));
			}
			id = "CN".concat(id);
		}
		result = queryById(id);
		System.out.println("结束查询1："+(System.currentTimeMillis()-ti));
		if (result.size()==0 && id.lastIndexOf(".") != -1) {
			id = id.substring(0, id.lastIndexOf("."));
			result = queryById(id);
		}
		System.out.println("结束查询2："+(System.currentTimeMillis()-ti));
		System.out.println("查询id: " + id);
		return result;
	}
	/**
	 * 查询论文信息
	 * @return
	 * Map<String,Object>
	 */
	public Map<String, Object> queryArticleInfo() {
		Map<String, Object> result = new HashMap<String, Object>();
		String resultJsonString=esMapper.getArticleInfo();
		System.out.println("查询出的字符串"+resultJsonString);
		result = parseArticleInfo(resultJsonString);
		return result;
	}
	
	/**
	 * 解析论文信息
	 * @param articleInfo
	 * @return
	 * Map<String,Object>
	 */
	private Map<String, Object> parseArticleInfo(String articleInfo) {
		Map<String, Object> result = new HashMap<String, Object>();
		return result;
	}

	/**
	 * id格式检验
	 * 
	 * @param name
	 * @param regx
	 * @return boolean
	 */
	private boolean isValidId(String name, String regx) {
		Pattern pattern = Pattern.compile(regx);
		return (name != null) && pattern.matcher(name.replaceAll(" ", "")).matches();
	}

	/**
	 * 根据id查询到所有结果
	 * @param id
	 * @return
	 * Map<String,Object>
	 */
	public Map<String, Object> queryById(String id) {
		// 需要返回的字段
		String fields = "[\"pubkind\",\"title\",\"legal_status_result\",\"appno\",\"appdate\",\"pubno\",\"pubdate\",\"legal_status\",\"granted_status\",\"patent_type\",\"ipcs\",\"agents\",\"inventors\",\"applicants\",\"owners\",\"abstract\"]";
		Map<String, Object> resultMap = new HashMap<String, Object>();
		Map<String, Object> result = new HashMap<String, Object>();
		JSONObject sourceJsonObject = null;
		String query = new String("{\"_source\":" + fields
				+ ",\"query\":{\"bool\":{\"must\":[{\"prefix\":{\"appno.keyword\":" + "\"" + id + "\"" + "}}]}}}");
		String resultString = esMapper.postQuery(query);
		if(StringUtils.isNotBlank(resultString)) {
			JSONObject hitJsonObject = JSONObject.parseObject(resultString).getJSONObject("hits");
			JSONArray hitsArray = (JSONArray) hitJsonObject.get("hits");
			if (hitsArray.size() > 0) {
				sourceJsonObject = hitsArray.getJSONObject(0).getJSONObject("_source");
				result = parseResult(sourceJsonObject);
				if(null!=result) {
					//查询成功，有结果返回
					resultMap.put("resonCode", "200");
					resultMap.put("result",result);
				}else {
					//查询成功，无结果返回，数据模型解析错误
					resultMap.put("resonCode", "2012");
					resultMap.put("result","");
				}
			}else {
				//查询成功，没有结果返回
				resultMap.put("resonCode", "2011");
				resultMap.put("result","");
			}
		}else {
			//1011 查询超时
			resultMap.put("resonCode", "1011");
			resultMap.put("result","");
		}	
		return resultMap;
	}

	private Map<String, Object> parseResult(JSONObject sourceJsonObject) {
		Map<String, Object> resultMap = new HashMap<String, Object>();
		String cnTitle = sourceJsonObject.getJSONObject("title").getString("text_en");
		String enTitle = sourceJsonObject.getJSONObject("title").getString("text_zh");
		resultMap.put("title", StringUtils.isNotBlank(cnTitle) ? cnTitle : enTitle);
		String application_number = sourceJsonObject.getString("appno");
		resultMap.put("application_number", application_number);
		String application_date1 = sourceJsonObject.getString("appdate");
		String application_date = "";
		if (StringUtils.isNotBlank(application_date1)) {
			application_date = application_date1.substring(0, 4) + "-" + application_date1.substring(4, 6) + "-"
					+ application_date1.substring(6, 8);
		}
		resultMap.put("application_date", application_date);
		String pubno = sourceJsonObject.getString("pubno");
		String pubkind = sourceJsonObject.getString("pubkind");
		JSONObject pmJsonObject = new JSONObject();
		pmJsonObject.put("pubno", StringUtils.isNotBlank(pubno)?pubno:"");
		pmJsonObject.put("pubkind", StringUtils.isNotBlank(pubkind)?pubkind:"");
		resultMap.put("publication_number", pmJsonObject);
		String publication_date1 = sourceJsonObject.getString("pubdate");
		String publication_date = "";
		if (StringUtils.isNotBlank(publication_date1)) {
			publication_date = publication_date1.substring(0, 4) + "-" + publication_date1.substring(4, 6) + "-"
					+ publication_date1.substring(6, 8);
		}
		resultMap.put("publication_date", publication_date); 
		JSONObject legalStatusJsonObject= sourceJsonObject.getJSONObject("legal_status_result");
		resultMap.put("legal_status", null!=legalStatusJsonObject?legalStatusJsonObject.getString("legalstatus"):"");
		resultMap.put("granted_status",null!=legalStatusJsonObject?legalStatusJsonObject.getString("granted_status"):"");
		String kind_code_desc = sourceJsonObject.getString("patent_type");
			resultMap.put("kind_code_desc", kind_code_desc);

		JSONArray IPC_list = sourceJsonObject.getJSONObject("ipcs").getJSONArray("mainclass");
		List<String> ipcNames = new ArrayList<String>();
		for (int j = 0; j < IPC_list.size(); j++) {
			String nameString=null!=IPC_list.getString(j)?PatentTypeEnum.parse(IPC_list.getString(j)).getMessage().replaceAll(" ", ""):"";
			ipcNames.add(nameString);
		}
		resultMap.put("IPC_list", IPC_list);
		resultMap.put("IPC_desc", ipcNames);
		String appcountry = StringUtils.isNoneBlank(sourceJsonObject.getString("appno"))
				? sourceJsonObject.getString("appno").substring(0, 2)
				: "";
		resultMap.put("appcountry", null!=TypeEnum.parse(appcountry)?TypeEnum.parse(appcountry).getMessage().replaceAll(" ", ""):"");

		JSONArray agengtObjectArray = sourceJsonObject.getJSONArray("agents");
		List<String> newAgengts = new ArrayList<String>();
		for (int i = 0; i < agengtObjectArray.size(); i++) {
			JSONObject inventorsObject = agengtObjectArray.getJSONObject(i);
			String agency = inventorsObject.getString("agency");
			newAgengts.add(agency);
		}
		resultMap.put("agency", newAgengts);
		JSONArray inventorObjectArray = sourceJsonObject.getJSONArray("inventors");
		List<String> newInventors = new ArrayList<String>();
		for (int i = 0; i < inventorObjectArray.size(); i++) {
			JSONObject inventorsObject = inventorObjectArray.getJSONObject(i);
			String nameString = null != inventorsObject ? StringUtils.isNotBlank(inventorsObject.getString("name_zh"))
					? inventorsObject.getString("name_zh")
					: inventorsObject.getString("name_en") : "";
			newInventors.add(nameString);
		}
		resultMap.put("inventors", newInventors);
		JSONArray applicantsArray = sourceJsonObject.getJSONArray("applicants");
		List<Map<String, Object>> newApplicants = new ArrayList<Map<String, Object>>();
		for (int i = 0; i < applicantsArray.size(); i++) {
			Map<String, Object> assigneesMap = new HashMap<String, Object>();
			JSONObject assigneeObject = applicantsArray.getJSONObject(i);
			assigneesMap.put("names",
					null != assigneeObject ? StringUtils.isNoneBlank(assigneeObject.getString("name_zh"))
							? assigneeObject.getString("name_zh")
							: assigneeObject.getString("name_en") : "");
			assigneesMap.put("province",
					null != assigneeObject
							? StringUtils.isNotBlank(assigneeObject.getJSONObject("address_zh").getString("province"))
									? assigneeObject.getJSONObject("address_zh").getString("province")
									: assigneeObject.getJSONObject("address_en").getString("province")
							: "");
			assigneesMap.put("city",
					null != assigneeObject
							? StringUtils.isNotBlank(assigneeObject.getJSONObject("address_zh").getString("city"))
									? assigneeObject.getJSONObject("address_zh").getString("city")
									: assigneeObject.getJSONObject("address_en").getString("city")
							: "");
			newApplicants.add(assigneesMap);
		}
		resultMap.put("assignees", newApplicants);

		JSONArray ownersObjectArray = sourceJsonObject.getJSONArray("owners");
		List<Map<String, Object>> newOwners = new ArrayList<Map<String, Object>>();
		for (int i = 0; i < ownersObjectArray.size(); i++) {
			JSONObject ownersObject = ownersObjectArray.getJSONObject(i);
			Map<String, Object> ownerMap = new HashMap<String, Object>();
			ownerMap.put("names",
					null != ownersObject ? StringUtils.isNotBlank(ownersObject.getString("name_zh"))
							? ownersObject.getString("name_zh")
							: ownersObject.getString("name_en") : "");
			ownerMap.put("province",
					null != ownersObject
							? StringUtils.isNoneBlank(ownersObject.getJSONObject("address_zh").getString("province"))
									? ownersObject.getJSONObject("address_zh").getString("province")
									: ownersObject.getJSONObject("address_en").getString("province")
							: "");
			ownerMap.put("city",
					null != ownersObject
							? StringUtils.isNoneBlank(ownersObject.getJSONObject("address_zh").getString("province"))
									? ownersObject.getJSONObject("address_zh").getString("city")
									: ownersObject.getJSONObject("address_en").getString("city")
							: "");
			newOwners.add(ownerMap);
		}
		if(newOwners.size()>0) {
			resultMap.put("owners", newOwners);
		}else {
			resultMap.put("owners", newApplicants);
		}
		String abstract1 = StringUtils.isNotBlank(sourceJsonObject.getJSONObject("abstract").getString("text_zh"))
				? sourceJsonObject.getJSONObject("abstract").getString("text_zh")
				: sourceJsonObject.getJSONObject("abstract").getString("text_en");
		resultMap.put("abstract", abstract1);
		return resultMap;
	}

}
